// -*- C++ -*-
/*!
 * @file  DummyMap.cpp
 * @brief Unit Test
 * @date $Date$
 *
 * $Id$
 */

#include "DummyMap.h"

// Module specification
// <rtc-template block="module_spec">
static const char* dummymap_spec[] =
  {
    "implementation_id", "DummyMap",
    "type_name",         "DummyMap",
    "description",       "Unit Test",
    "version",           "1.0.0",
    "vendor",            "Hiroaki Yaguchi, JSK",
    "category",          "Test",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
DummyMap::DummyMap(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_mapOut("map", m_map),
    m_mapconfigOut("mapconfig", m_mapconfig),
    m_boxOut("box", m_box),
    m_pointsOut("points", m_points)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
DummyMap::~DummyMap()
{
}



RTC::ReturnCode_t DummyMap::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("map", m_mapOut);
  addOutPort("mapconfig", m_mapconfigOut);
  addOutPort("box", m_boxOut);
  addOutPort("points", m_pointsOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DummyMap::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DummyMap::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DummyMap::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t DummyMap::onActivated(RTC::UniqueId ec_id)
{
  m_map.width=10;
  m_map.height=10;
  m_map.cells.length(100);
  for(int i=0;i<10;i++){
    for(int j=0;j<10;j++){
      //m_map.cells[i*10+j]=50;
      m_map.cells[i*10+j]=(i+1)*(j+1);
    }
  }

  m_mapconfig.xScale=100.0;
  m_mapconfig.yScale=100.0;
  m_mapconfig.width=10;
  m_mapconfig.height=10;
  m_mapconfig.origin.position.x=0.0;
  m_mapconfig.origin.position.y=0.0;
  m_mapconfig.origin.heading=0.0;

  float x,y,z;
  int idx;
  m_points.data.length(300);
  z=100.0;
  for(int i=0;i<10;i++){
    y=(i-5)*100.0-500.0;
    for(int j=0;j<10;j++){
      x=(j-5)*100.0;
      idx=3*(i*10+j);
      m_points.data[idx]=x;
      m_points.data[idx+1]=y;
      m_points.data[idx+2]=z;
    }
  }

  m_box.data.length(9);
  m_box.data[0]=0.0;
  m_box.data[1]=500.0;
  m_box.data[2]=150.0;

  m_box.data[3]=0.0;
  m_box.data[4]=0.0;
  m_box.data[5]=1.05;

  m_box.data[6]=300.0;
  m_box.data[7]=300.0;
  m_box.data[8]=300.0;

  isfirst=true;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t DummyMap::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t DummyMap::onExecute(RTC::UniqueId ec_id)
{
  if(isfirst){
    m_mapconfigOut.write();
    isfirst=false;
  }
  m_mapOut.write();
  m_pointsOut.write();

  m_box.data[5]+=0.01;
  m_box.data[5]-=(int)(m_box.data[5]/6.283184)*6.283184;
  m_boxOut.write();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DummyMap::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DummyMap::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DummyMap::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DummyMap::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DummyMap::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DummyMapInit(RTC::Manager* manager)
  {
    coil::Properties profile(dummymap_spec);
    manager->registerFactory(profile,
                             RTC::Create<DummyMap>,
                             RTC::Delete<DummyMap>);
  }
  
};


