// -*- C++ -*-
/*!
 * @file  GTKPickview.h
 * @brief 3D Viewer
 * @date  $Date$
 *
 * $Id$
 */

#ifndef GTKPICKVIEW_H
#define GTKPICKVIEW_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>

#include "GtkView.h"

using namespace RTC;

/*!
 * @class GTKPickview
 * @brief 3D Viewer
 *
 */
class GTKPickview
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  GTKPickview(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~GTKPickview();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  pthread_mutex_t *mutex(){return &thread_lock;}
  bool threadRunning(){return thread_running;}
  void standbyThread(){thread_running=true;}
  void destroyThread(){thread_running=false;}
  pthread_t getThreadID(){return thread_id;}
  void setThreadID(pthread_t newid){thread_id=newid;}
  GtkView *createGtkView(){
    gview=new GtkView();
    return gview;
  }
  GtkView *getGtkView(){return gview;}

  //! @brief set GLObject directry (debug code)
  void setGLObject(GLObject *o){glo=o;}
  //! @brief get GLObject directry (debug code)
  GLObject *getGLObject(){return glo;}

  void createGLObjects();
  GLGrid *getGLGrid(){return gl_grid;}
  GLFeaturePoints *getGLFeaturePoints(){return gl_points;}
  GLBox *getGLBox(){return gl_box;}
  GLVoxel *getGLVoxel(){return gl_voxel;}

  void setPoints(vector<float> &pts);
  void setPointsColor(vector<unsigned char> &col);
  void setMap(int width,int height,vector<float> &map);
  void setMapConfig(float xs,float ys,Pose2D &pose);
  void setBox(vector<float> &box);
  void getPoints(vector<float> &pts);
  void getPointsColor(vector<unsigned char> &col);
  void getMap(vector<float> &map);
  int getMapWidth(){return map_width;}
  int getMapHeight(){return map_height;}
  float getMapXScale(){return map_xscale;}
  float getMapYScale(){return map_yscale;}
  float getMapPosX(){return (float)map_pose.position.x;}
  float getMapPosY(){return (float)map_pose.position.y;}
  float getMapRot(){return (float)map_pose.heading;}
  void getBox(vector<float> &box);
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  int m_PointCloudColor;
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  //TimedFloatSeq m_Map2D;
  OGMapTile m_Map2D;
  /*!
   */
  //InPort<TimedFloatSeq> m_Map2DIn;
  InPort<OGMapTile> m_Map2DIn;

  OGMapConfig m_MapConfig;
  /*!
   */
  InPort<OGMapConfig> m_MapConfigIn;

  TimedFloatSeq m_Box3D;
  /*!
   */
  InPort<TimedFloatSeq> m_Box3DIn;

  PointCloud m_Points3D;
  /*!
   */
  InPort<PointCloud> m_Points3DIn;

  // for Kinect
  CameraImage m_ColorImage;
  /*!
   */
  InPort<CameraImage> m_ColorImageIn;
  CameraImage m_DepthImage;
  /*!
   */
  InPort<CameraImage> m_DepthImageIn;

  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  pthread_mutex_t thread_lock;
  pthread_t thread_id;
  bool thread_running;
  GtkView *gview;
  GLObject *glo;

  GLGrid *gl_grid;
  GLBox *gl_box;
  GLVoxel *gl_voxel;
  GLFeaturePoints *gl_points;

  vector<float> vec_points;
  vector<unsigned char> vec_pointscolor;
  vector<float> vec_map;
  vector<float> vec_box;
  int map_width,map_height;
  float map_xscale,map_yscale;
  Pose2D map_pose;

  // </rtc-template>

  // <rtc-template block="private_operation">

  // </rtc-template>

};

// for pthread
void startGTKThread(GTKPickview *gp);
void stopGTKThread(GTKPickview *gp);
void *GTKThreadRoutine(void *arg);
static gboolean polling_timer_event(gpointer p);


extern "C"
{
  DLL_EXPORT void GTKPickviewInit(RTC::Manager* manager);
};

#endif // GTKPICKVIEW_H
